/*global window */

function extend(subClass, superClass) {
    'use strict';

    var F = function () {};
    F.prototype = superClass.prototype;
    subClass.prototype = new F();
    subClass.prototype.constructor = subClass;
    subClass.superclass = superClass.prototype;
    if (superClass.prototype.constructor === Object.prototype.constructor) {
        superClass.prototype.constructor = superClass;
    }
}

var Chat = (function () {
        'use strict';

        function Chat(container_id) {
            var container = window.document.getElementById(container_id),
                table = window.document.createElement("table"),
                message_input = window.document.createElement("input"),
                message_send_btn = window.document.createElement("button"),
                self = this;

            while (container.firstChild) {
                container.removeChild(container.firstChild);
            }

            container.appendChild(message_input);
            message_send_btn.appendChild(window.document.createTextNode("Send"));
            container.appendChild(message_send_btn);
            container.appendChild(table);
            (function () {
                // Setup table headers
                var tr = window.document.createElement("tr"),
                    th1 = window.document.createElement("th"),
                    th2 = window.document.createElement("th");

                table.appendChild(tr);
                tr.appendChild(th1);
                tr.appendChild(th2);
                th1.appendChild(window.document.createTextNode("Client"));
                th2.appendChild(window.document.createTextNode("Message"));
            }());

            // Send message
            message_send_btn.addEventListener("click", function () {
                var message = message_input.value;
                self.send(message);
            });

            this.disconnect = function () { return this; };
            this.send = function () { return this; };
            this.show_new_message = function (from, message) {
                var tr = window.document.createElement("tr"),
                    td1 = window.document.createElement("td"),
                    td2 = window.document.createElement("td");

                table.appendChild(tr);
                tr.appendChild(td1);
                tr.appendChild(td2);
                td1.appendChild(window.document.createTextNode(from));
                td2.appendChild(window.document.createTextNode(message));
            };
        }
        return Chat;
    }()),

    WSChat = (function () {
        'use strict';

        function WSChat(container_id, server_url, name) {
            Chat.call(this, container_id);
            var connection = new window.WebSocket(server_url),
                self = this;

            // On connection
            connection.onopen = function () {
                var connect_data = {type: "connect", name: name};
                connection.send(JSON.stringify(connect_data));
            };

            // On message
            connection.onmessage = function (e) {
                var message = e.data,
                    parsed_message = JSON.parse(message);

                switch (parsed_message.type) {
                case "new_message":
                    // Show new message
                    self.show_new_message(parsed_message.from, parsed_message.message);
                    break;
                }
            };

            this.send = function (message) { // OVERRIDE
                var data = {
                    type: "send_message",
                    message: message
                };
                connection.send(JSON.stringify(data));
                return this;
            };
            this.disconnect = function () { // OVERRIDE
                connection.close();
                return this;
            };
        }

        extend(WSChat, Chat);
        return WSChat;
    }());