/*global module */
(function () {
    'use strict';

    function Interface(name, methods) {
        if (arguments.length !== 2) {
            throw new Error("Interface constructor called with "
                + arguments.length + "arguments, but expected exactly 2.");
        }
        this.name = name;
        this.methods = [];
        var i, len;
        for (i = 0, len = methods.length; i < len; i += 1) {
            if (typeof methods[i] !== 'string') {
                throw new Error(
                    "Interface constructor expects method names to be "
                        + "passed in as a string."
                );
            }
            this.methods.push(methods[i]);
        }
    }

    Interface.ensureImplements = function (object) {
        if (arguments.length < 2) {
            throw new Error("Function Interface.ensureImplements called with "
                + arguments.length + "arguments, but expected at least 2.");
        }
        var i, len, interf, j, methodsLen, method;
        for (i = 1, len = arguments.length; i < len; i += 1) {
            interf = arguments[i];
            if (interf.constructor !== Interface) {
                throw new Error("Function Interface.ensureImplements expects arguments"
                    + "two and above to be instances of Interface.");
            }
            for (j = 0, methodsLen = interf.methods.length; j < methodsLen; j += 1) {
                method = interf.methods[j];
                if (!object[method] || typeof object[method] !== 'function') {
                    throw new Error("Function Interface.ensureImplements: object "
                        + "does not implement the " + interf.name
                        + " interface. Method " + method + " was not found.");
                }
            }
        }
    };

    module.exports.Interface = Interface;
}());