/*global require, console */

function extend(subClass, superClass) {
    'use strict';

    var F = function () {};
    F.prototype = superClass.prototype;
    subClass.prototype = new F();
    subClass.prototype.constructor = subClass;
    subClass.superclass = superClass.prototype;
    if (superClass.prototype.constructor === Object.prototype.constructor) {
        superClass.prototype.constructor = superClass;
    }
}

var Interface = require("./Interface.js").Interface,

    IChatClient = new Interface('IChatClient', ['receive_message', 'name']),

    http = require('http'),

    // HTTP Server
    http_server = http.createServer(function (req, res) {
        var req_url = "/" === String(req.url) ? "/index.html" : req.url,
            client_dir = "../client/";

        require('fs').readFile(client_dir + req_url,
            function (err, data) {
                if (err) {
                    res.writeHead(500);
                    return res.end(data);
                }
                res.writeHead(200);
                return res.end(data);
            });
    }).listen(8123, 'localhost'),

    WSChatClient = (function () { // Implements IChatClient
        'use strict';

        function WSChatClient(name, connection) {
            this.name = function () { return name; };
            this.receive_message = function (message, from) {
                Interface.ensureImplements(from, IChatClient);
                var data = {
                    type: "new_message",
                    message: message,
                    from: from.name()
                };
                connection.sendUTF(JSON.stringify(data));
                return this;
            };
        }
        return WSChatClient;
    }()),

    ChatServer = (function () {
        'use strict';

        function ChatServer() {
            var clients = [];

            this.connect_client = function (new_client) {
                Interface.ensureImplements(new_client, IChatClient);
                clients.push(new_client);
                return this;
            };
            this.disconnect_client = function (client) {
                Interface.ensureImplements(client, IChatClient);
                clients =
                    clients.
                    filter(function (c) { return c !== client; });
                return this;
            };
            this.broadcast_message = function (client, message) {
                Interface.ensureImplements(client, IChatClient);
                clients.
                    forEach(function (c) { c.receive_message(message, client); });
                return this;
            };
        }

        return ChatServer;
    }()),

    WSChatServer = (function () {
        'use strict';

        function WSChatServer(http_server) {
            WSChatServer.superclass.constructor.call(this);
            var self = this,
                WebSocketServer = require('websocket').server,
                ws_server = new WebSocketServer({httpServer: http_server, autoAcceptConnections: true});

            ws_server.on('connect', function (connection) {
                var chat_client;

                connection.on('close', function () {
                    self.disconnect_client(chat_client);
                    console.log("client leaving: " + chat_client.name());
                });

                connection.on('message', function (message) {
                    if (message.type === 'utf8') {

                        var parsed_message = null,
                            message_type;
                        try {
                            parsed_message = JSON.parse(message.utf8Data);
                        } catch (e) { return; }

                        message_type = parsed_message.type;

                        switch (message_type) {
                        case "connect":
                            if (chat_client instanceof WSChatClient === false) {
                                chat_client = new WSChatClient(parsed_message.name, connection);
                                self.connect_client(chat_client);
                                console.log("new client: " + chat_client.name());
                            }
                            break;
                        case "send_message":
                            self.broadcast_message(chat_client, parsed_message.message);
                            console.log("new message: " + parsed_message.message + ", from: " + chat_client.name());
                            break;
                        }
                    }
                });
            });
        }
        extend(WSChatServer, ChatServer);
        return WSChatServer;
    }()),

    chat_server = new WSChatServer(http_server);